/**
 * Copyright French Prime minister Office/SGMAP/DINSIC/Vitam Program (2019-2020)
 * and the signatories of the "VITAM - Accord du Contributeur" agreement.
 *
 * contact@programmevitam.fr
 *
 * This software is a computer program whose purpose is to implement
 * implement a digital archiving front-office system for the secure and
 * efficient high volumetry VITAM solution.
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package fr.gouv.vitamui.commons.api.domain;

import java.io.Serializable;
import java.util.Collection;
import java.util.Map;

/**
 * Common class which can be used to return a paginated list of values and aggregation results.
 *
 * @param <T> The class to be returned, it should be a Dto class.
 */
public class ResultsDto<T> implements Serializable {

    private transient Collection<T> values;

    private int pageNum;

    private int pageSize;

    private boolean hasMore;

    private Map<String, Object> groups;

    public ResultsDto() {
        // Intentionally empty.
    }

    public ResultsDto(final Collection<T> values, final int pageNum, final int maxResults,
            final boolean hasMore) {
        this.values = values;
        this.pageNum = pageNum;
        this.pageSize = maxResults;
        this.hasMore = hasMore;
    }

    public ResultsDto(final Collection<T> values, final int pageNum, final int maxResults,
            final boolean hasMore, Map<String, Object> groups) {
        this.values = values;
        this.pageNum = pageNum;
        this.pageSize = maxResults;
        this.hasMore = hasMore;
        this.groups = groups;
    }

    public ResultsDto(final Collection<T> values, ResultsDto resultsDto) {
        this.values = values;
        this.pageNum = resultsDto.getPageNum();
        this.pageSize = resultsDto.getPageSize();
        this.hasMore = resultsDto.isHasMore();
        this.groups = resultsDto.getGroups();
    }

    public ResultsDto(final PaginatedValuesDto<T> paginatedValuesDto) {
        this.values = paginatedValuesDto.getValues();
        this.pageNum = paginatedValuesDto.getPageNum();
        this.pageSize = paginatedValuesDto.getPageSize();
        this.hasMore = paginatedValuesDto.isHasMore();
    }

    public int getPageNum() {
        return pageNum;
    }

    public int getPageSize() {
        return pageSize;
    }

    public boolean isHasMore() {
        return hasMore;
    }

    public void setHasMore(final boolean hasMore) {
        this.hasMore = hasMore;
    }

    public void setPageNum(final int pageNum) {
        this.pageNum = pageNum;
    }

    public void setPageSize(final int pageSize) {
        this.pageSize = pageSize;
    }

    public Collection<T> getValues() {
        return values;
    }

    public Map<String, Object> getGroups() {
        return groups;
    }

}
